function V = get(U,F)

  % LINFACTORDISCR.GET
	
  switch F
   case 'weights'
	V = U.weights;
   case 'bias'
	V = U.bias;
   case 'params'
	V = [U.weights(:)' U.bias(:)'];
   case 'numParams'
	V = numel(U.weights) + numel(U.bias);
   otherwise
	V = get(U.factordiscr,F);
  end;
